/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2013 Raytrix GmbH. All rights reserved.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma once

#pragma managed(push, off)

#include "ECallbackMode.h"
#include "EImageTransferMode.h"
#include "ECamProperty.h"
#include "ECamTriggerMode.h"
#include "EWhiteImageStatus.h"
#include "EImageFormat.h"
#include "ECompressionMode.h"
#include "ECamVideoMode.h"
#include "EOisMode.h"
#include "EBinningMode.h"

#include "IImageProperty.h"

#ifdef LINUX
	#define __cdecl __attribute__((__cdecl__))
#endif

namespace Rx
{
	namespace RayCam
	{
		/// <summary> This is the callback functions which returns the captured images. </summary>
		typedef void (__cdecl * TCallbackFunction)(void* callBackOwner, unsigned int iWidth, unsigned int iHeight,
				unsigned int iPixelFormat,
				unsigned int iDataFormat, void* pDataPtr);

		/// <summary> This is the callback functions which returns the captured images. </summary>
		typedef void (__cdecl * TCallbackFunctionExtended)(void* callBackOwner, unsigned int iWidth, unsigned int iHeight,
				unsigned int iPixelFormat,
				unsigned int iDataFormat, void* pDataPtr, IImageProperty* pImageData);

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// <summary>
		/// 	This class represents a camera. To use the camera in streaming mode:
		/// 	    1. Call Rx::RayCam::IDevice::Open() to initialize the camera (Can take up to 25 seconds).
		/// 	    2. Register your callback function Rx::RayCam::TCallbackFunction using Rx::RayCam::IDevice::RegisterCallback().
		/// 	    3. Call Rx::RayCam::IDevice::StartStream(), to start the video stream.
		/// 	    4. The callback function gets called with every new image.
		/// </summary>
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		class IDevice
		{
		public:

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Opens this camera device. Might throw a firmware mismatch exception which indicates that either the firmware of
			/// 	the camera or the firmware on your hard disk is newer. In this case, you should upgrade or downgrade the firmware
			/// 	to get a fully functional camera. Otherwise, if you call Open() again, the camera might me able to open but it's very
			/// 	likely for the camera to not work at some point.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void Open(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Closes the camera and disconnects it.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void Close(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Resets the camera.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void Reset(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Registers the receive image callback specified by Rx::RayCam::TCallbackFunction.
			/// </summary>
			///
			/// <param name="callBackOwner">	 [in,out] If non-null, the owner of the call back. </param>
			/// <param name="pCallbackFunction"> The callback function. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void RegisterCallback(void* callBackOwner, TCallbackFunction pCallbackFunction) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Registers the extended receive image callback specified by Rx::RayCam::TCallbackFunction.
			/// </summary>
			///
			/// <param name="callBackOwner">	 [in,out] If non-null, the owner of the call back. </param>
			/// <param name="pCallbackFunction"> The callback function. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void RegisterCallback(void* callBackOwner, TCallbackFunctionExtended pCallbackFunction) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Unregisters the receive image callback.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void UnregisterCallback(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Unregisters the extended receive image callback.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void UnregisterCallbackExtended(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Starts a stream in a specific mode.
			/// </summary>
			///
			/// <param name="eMode"> The Rx::RayCam::ECamTriggerMode::ID trigger mode. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void StartStream(Rx::RayCam::ECamTriggerMode::ID eMode) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Starts a video stream but suspended. You won't receive any images until you call Rx::RayCam::IDevice::SuspendStream()
			/// 	with false.
			/// </summary>
			///
			/// <param name="eMode"> The Rx::RayCam::ECamTriggerMode::ID trigger mode. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void StartStreamSuspended(Rx::RayCam::ECamTriggerMode::ID eMode) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Stops a stream.
			/// 	The function abort pending image exposures.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void StopStream(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Suspend or unsuspend the camera.
			///
			/// 	Unsuspend the camera before you call Rx::RayCam::IDevice::StopStream()!
			/// </summary>
			///
			/// <param name="bSuspend"> true to suspend camera, false to unsuspend. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SuspendStream(bool bSuspend) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Triggers this camera to deliver an image via the callback function Rx::RayCam::TCallbackFunction (only in
			/// 	Rx::RayCam::ECamTriggerMode::SoftwareSingle mode).
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void Trigger(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a camera property of type int or bool (0 represents false).
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, int& theValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a camera property of type double.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, double& theValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a camera property of type long long (0 represents false).
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, long long& theValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets the min and max value of a camera property of type int.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue1"> [out] The first value. </param>
			/// <param name="theValue2"> [out] The second value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, int& theValue1, int& theValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets the min and max value of a camera property of type double.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue1"> [out] The first value. </param>
			/// <param name="theValue2"> [out] The second value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, double& theValue1, double& theValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets the min and max value of a camera property of type double.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue1"> [out] The first value. </param>
			/// <param name="theValue2"> [out] The second value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, long long& theValue1, long long& theValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a camera property of type void*.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/// <param name="theSize">  [in,out] Size of the. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, void* theValue, int& theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Writes a char array into the theValue buffer. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  [in,out] The size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, char* theValue, int& theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Writes a char array into the theValue buffer. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  [in,out] The size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, wchar_t* theValue, int& theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets the min and max value of a camera property of type int.
			/// </summary>
			///
			/// <param name="eProp">  The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theMin"> [in,out] The min value. </param>
			/// <param name="theMax"> [in,out] The max value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetPropertyRange(ECamProperty::ID eProp, int& theMin, int& theMax) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets the min and max value of a camera property of type double.
			/// </summary>
			///
			/// <param name="eProp">  The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theMin"> [in,out] The min value. </param>
			/// <param name="theMax"> [in,out] The max value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetPropertyRange(ECamProperty::ID eProp, double& theMin, double& theMax) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets the min and max value of a camera property of type double.
			/// </summary>
			///
			/// <param name="eProp">  The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theMin"> [in,out] The min value. </param>
			/// <param name="theMax"> [in,out] The max value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetPropertyRange(ECamProperty::ID eProp, long long& theMin, long long& theMax) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Check if a property is available for Get/Set operations.
			/// </summary>
			///
			/// <param name="eProp"> The Rx::RayCam::ECamProperty::ID property. </param>
			///
			/// <returns> True if it the property is available. </returns>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual bool HasProperty(ECamProperty::ID eProp) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property of type int or bool (0 represents false).
			/// </summary>
			///
			/// <param name="eProp">    The property. </param>
			/// <param name="newValue"> The new value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(ECamProperty::ID eProp, int newValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property of type double.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="newValue"> The new value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(ECamProperty::ID eProp, double newValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property of type long long.
			/// </summary>
			///
			/// <param name="eProp">    The property. </param>
			/// <param name="newValue"> The new value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(ECamProperty::ID eProp, long long newValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property with two values of type int or bool (0 represents false).
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="newValue1"> The new value1. </param>
			/// <param name="newValue2"> The new value2. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(ECamProperty::ID eProp, int newValue1, int newValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property with two values of type double.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="newValue1"> The new value1. </param>
			/// <param name="newValue2"> The new value2. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(ECamProperty::ID eProp, double newValue1, double newValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property with two values of type double.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="newValue1"> The new value1. </param>
			/// <param name="newValue2"> The new value2. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(ECamProperty::ID eProp, long long newValue1, long long newValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property of type void*.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/// <param name="theSize">  Size of the. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(ECamProperty::ID eProp, void* theValue, int theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property of type char*. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  The size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(ECamProperty::ID eProp, char* theValue, int theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property of type char*. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  The size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(ECamProperty::ID eProp, wchar_t* theValue, int theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property with two values of type int or bool (0 represents false).
			/// </summary>
			///
			/// <param name="eProp">  The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="newMin"> The min value. </param>
			/// <param name="newMax"> The max value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetPropertyRange(ECamProperty::ID eProp, int newMin, int newMax) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property with two values of type double.
			/// </summary>
			///
			/// <param name="eProp">  The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="newMin"> The min value. </param>
			/// <param name="newMax"> The max value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetPropertyRange(ECamProperty::ID eProp, double newMin, double newMax) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a camera property with two values of type double.
			/// </summary>
			///
			/// <param name="eProp">  The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="newMin"> The min value. </param>
			/// <param name="newMax"> The max value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetPropertyRange(ECamProperty::ID eProp, long long newMin, long long newMax) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Try set camera configuration.
			/// </summary>
			///
			/// <param name="dMinFps">	    [out] The minimum FPS. </param>
			/// <param name="dMaxFPS">	    [out] The maximum FPS. </param>
			/// <param name="dMinExposure"> [out] The minimum exposure. </param>
			/// <param name="dMaxExposure"> [out] The maximum exposure. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void TestCameraConfiguration(double& dMinFps, double& dMaxFPS, double& dMinExposure, double& dMaxExposure) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	This will test the test configuration.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void TestCameraConfiguration(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	This will apply the set up test configuration.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void ApplyCameraConfiguration(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Take a white image and do color balancing.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void ProcessWhiteBalance(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Take a white image and do devignetting. This will remove vignetting artifacts.
			/// </summary>
			///
			/// <returns> The Rx::RayCam::EWhiteImageStatus::ID status of the white image. </returns>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual EWhiteImageStatus::ID ProcessDevignetting(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Locks all Texture operations when using OpenGl Texture Transfer
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void LockTextureAccess() = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Unlocks the texture mutex when using OpenGL Texture Transfer
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void UnlockTextureAccess() = 0;
		};
	}
}

#pragma managed(pop)
