/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2013 Raytrix GmbH. All rights reserved.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma once

#pragma managed(push, off)

#include "EDriverProperty.h"
#include "IDevice.h"

#include "Rx.Interop.Runtime31/IException.h"

namespace Rx
{
	namespace RayCam
	{
		/// <summary> This is the error callback function. </summary>
		typedef void (__cdecl * TErrorCallback)(void* callBackOwner, Rx::Interop::Runtime31::IException& ex, Rx::RayCam::IDevice* throwingDevice);

		/// <summary> Defines a callback function for new devices. </summary>
		typedef void (__cdecl * TCallbackFunctionPnpConnect)(void* callBackOwner, IDevice* newDevice);

		/// <summary> Defines a callback function for removed devices. </summary>
		typedef void (__cdecl * TCallbackFunctionPnpDisconnect)(void* callBackOwner, IDevice* removedDevice);

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// <summary>
		/// 	Driver class initializes the driver and checks for connected and valid devices. You can search for devices, create or
		/// 	delete devices.
		/// </summary>
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		class IDriver
		{
		public:

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Initializes the driver. Only in case of success it is safe to proceed with e.g. DiscoverDevices(iDeviceCount).
			///
			/// 	\pre Rx::RayCam::CreateDriver() must be called first.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void InitDriver(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Initializes the driver. Only in case of success it is safe to proceed with e.g. DiscoverDevices(iDeviceCount).
			///
			///
			/// 	\pre Rx::RayCam::CreateDriver() must be called first.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void InitDriver(void* pvErrorCallBackOwner, TErrorCallback fErrorCallback) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Discovers devices and returns the total count of all devices.
			///
			/// 	\pre Rx::RayCam::IDriver::InitDriver() must be called first.
			/// </summary>
			///
			/// <param name="iDeviceCount"> [in,out] Number of devices. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void DiscoverDevices(int& iDeviceCount) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Discover devices asynchronous. For each Device discovered, you will receive a callback invocation of
			/// 	the Plug and Play Connect callback (use  RegisterPnPConnectCallback() to register a callback)
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void DiscoverDevicesAsync() = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Creates a device. Index represents the device attachment order.
			///
			/// 	\pre Rx::RayCam::IDriver::DiscoverDevices() must be called first.
			/// </summary>
			///
			/// <param name="iIdx"> Index, starting with zero. </param>
			///
			/// <returns> The new Rx::RayCam::IDevice device. </returns>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual IDevice* CreateDevice(int iIdx) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Creates a devic with the given serial number.
			///
			/// 	\pre Rx::RayCam::IDriver::DiscoverDevices() must be called first.
			/// </summary>
			///
			/// <param name="pcSerial"> [in] The serial number of the camera to create. </param>
			///
			/// <returns> The new Rx::RayCam::IDevice device. </returns>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual IDevice* CreateDevice(const char* pcSerial) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Destroys the Rx::RayCam::IDevice described by pDevice.
			/// </summary>
			///
			/// <param name="pDevice"> [in,out] The device. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void DestroyDevice(IDevice* pDevice) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Registers the error callback.
			/// </summary>
			///
			/// <param name="callBackOwner">  [in] If non-null, the owner of the callback. </param>
			/// <param name="fErrorCallback"> The error callback. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void RegisterErrorCallback(void* callBackOwner, TErrorCallback fErrorCallback) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Unregisters the error callback.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void UnregisterErrorCallback(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Registers the callback pnp connect. After registering connect callback, you also have to register the disconnect
			/// 	callback. If you register a PnP callback function, you'll get a callback each time a new camera has been connected
			/// 	computer.
			/// </summary>
			///
			/// <param name="callBackOwner">	 [in,out] If non-null, the owner of the call back. </param>
			/// <param name="pCallbackFunction"> The callback function. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void RegisterPnpConnectCallback(void* callBackOwner, TCallbackFunctionPnpConnect pCallbackFunction) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Unregisters the callback pnp connect.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void UnregisterPnpConnectCallback(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Registers the callback pnp disconnect. After registering disconnect callback, you also have to register the connect
			/// 	callback.If you register a PnP callback function, you'll get a callback each time a new camera has been removed from
			/// 	the computer. You also HAVE to ensure that every device provided by the disconnect callback will be closed and also
			/// 	given back to the driver.
			/// </summary>
			///
			/// <param name="callBackOwner">	 [in,out] If non-null, the owner of the call back. </param>
			/// <param name="pCallbackFunction"> The callback function. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void RegisterPnpDisconnectCallback(void* callBackOwner, TCallbackFunctionPnpDisconnect pCallbackFunction) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Unregisters the callback pnp disconnect.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void UnregisterPnpDisconnectCallback(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(EDriverProperty::ID eProp, int& theValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a driver property of type double.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(EDriverProperty::ID eProp, double& theValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(EDriverProperty::ID eProp, long long& theValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue">  [in,out] The value. </param>
			/// <param name="theValue2"> [in,out] The second the value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(EDriverProperty::ID eProp, int& theValue, int& theValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a driver property of type double.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue">  [in,out] The value. </param>
			/// <param name="theValue2"> [in,out] The second the value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(EDriverProperty::ID eProp, double& theValue, double& theValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue">  [in,out] The value. </param>
			/// <param name="theValue2"> [in,out] The second the value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(EDriverProperty::ID eProp, long long& theValue, long long& theValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Writes a char array into the theValue buffer. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  [in,out] Size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(EDriverProperty::ID eProp, void* theValue, int& theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Writes a char array into the theValue buffer. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  [in,out] Size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(EDriverProperty::ID eProp, char* theValue, int& theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Writes a char array into the theValue buffer. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  [in,out] Size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(EDriverProperty::ID eProp, wchar_t* theValue, int& theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">	   The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newMinValue"> [in,out] The value. </param>
			/// <param name="newMaxValue"> [in,out] The new maximum value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetPropertyRange(EDriverProperty::ID eProp, int& newMinValue, int& newMaxValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a driver property of type double.
			/// </summary>
			///
			/// <param name="eProp">	   The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newMinValue"> [in,out] The value. </param>
			/// <param name="newMaxValue"> [in,out] The new maximum value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetPropertyRange(EDriverProperty::ID eProp, double& newMinValue, double& newMaxValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">	   The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newMinValue"> [in,out] The value. </param>
			/// <param name="newMaxValue"> [in,out] The new maximum value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetPropertyRange(EDriverProperty::ID eProp, long long& newMinValue, long long& newMaxValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Check if a property is available for Get/Set operations.
			/// </summary>
			///
			/// <param name="eProp"> The Rx::RayCam::EDriverProperty::ID property. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void HasProperty(EDriverProperty::ID eProp) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newValue"> The new value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(EDriverProperty::ID eProp, int newValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a driver property of type double.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newValue"> The new value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(EDriverProperty::ID eProp, double newValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newValue"> The new value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(EDriverProperty::ID eProp, long long newValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newValue">  The new value. </param>
			/// <param name="newValue2"> The second new value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(EDriverProperty::ID eProp, int newValue, int newValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a driver property of type double.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newValue">  The new value. </param>
			/// <param name="newValue2"> The second new value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(EDriverProperty::ID eProp, double newValue, double newValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newValue">  The new value. </param>
			/// <param name="newValue2"> The second new value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(EDriverProperty::ID eProp, long long newValue, long long newValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Writes a char array into the theValue buffer. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  Size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(EDriverProperty::ID eProp, void* theValue, int theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Writes a char array into the theValue buffer. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  Size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(EDriverProperty::ID eProp, char* theValue, int theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Writes a char array into the theValue buffer. The size of the buffer has to be in theSize.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value buffer. </param>
			/// <param name="theSize">  Size of the buffer. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetProperty(EDriverProperty::ID eProp, wchar_t* theValue, int theSize) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a driver property of type int.
			/// </summary>
			///
			/// <param name="eProp">	   The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newMinValue"> The new value. </param>
			/// <param name="newMaxValue"> The new maximum value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetPropertyRange(EDriverProperty::ID eProp, int newMinValue, int newMaxValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a driver property of type double.
			/// </summary>
			///
			/// <param name="eProp">	   The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newMinValue"> The new value. </param>
			/// <param name="newMaxValue"> The new maximum value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetPropertyRange(EDriverProperty::ID eProp, double newMinValue, double newMaxValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Sets a driver property of type double.
			/// </summary>
			///
			/// <param name="eProp">	   The Rx::RayCam::EDriverProperty::ID property. </param>
			/// <param name="newMinValue"> The new value. </param>
			/// <param name="newMaxValue"> The new maximum value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void SetPropertyRange(EDriverProperty::ID eProp, long long newMinValue, long long newMaxValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Searches for updates and updates the camera firmware. This function call can take up to 2 minutes.
			///
			/// 	\attention Do NOT disconnect the camera while your program is inside this function.
			///
			/// 	\attention You can use this function only with one camera connected to your computer.
			/// </summary>
			///
			/// <param name="pxCamera"> [in] The camera you want to update. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void UpdateFirmware(Rx::RayCam::IDevice* pxCamera) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Checks for updates.
			///
			/// 	\pre Rx::RayCam::IDevice::Open() must be called first.
			/// </summary>
			///
			/// <param name="pxCamera"> [in] The camera you want to update. </param>
			///
			/// <returns>
			/// 	-1 if a firmware downgrade is available, 0 if disk and firmware are the same, 1 if a firmware upgrade is available.
			/// </returns>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual int UpdateCheck(Rx::RayCam::IDevice* pxCamera) = 0;
		};
	}
}

#pragma managed(pop)
